#!/usr/bin/python
# Copyright (c) 2018, Fran Fitzpatrick <francis.x.fitzpatrick@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_config
author: Fran Fitzpatrick (@fxfitz)
short_description: Manage Global FreeIPA Configuration Settings
description:
  - Modify global configuration settings of a FreeIPA Server.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  ipaconfigstring:
    description: Extra hashes to generate in password plug-in.
    aliases: ["configstring"]
    type: list
    elements: str
    choices: ["AllowNThash", "KDC:Disable Last Success", "KDC:Disable Lockout", "KDC:Disable Default Preauth for SPNs"]
    version_added: '2.5.0'
  ipadefaultloginshell:
    description: Default shell for new users.
    aliases: ["loginshell"]
    type: str
  ipadefaultemaildomain:
    description: Default e-mail domain for new users.
    aliases: ["emaildomain"]
    type: str
  ipadefaultprimarygroup:
    description: Default group for new users.
    aliases: ["primarygroup"]
    type: str
    version_added: '2.5.0'
  ipagroupobjectclasses:
    description: A list of group objectclasses.
    aliases: ["groupobjectclasses"]
    type: list
    elements: str
    version_added: '7.3.0'
  ipagroupsearchfields:
    description: A list of fields to search in when searching for groups.
    aliases: ["groupsearchfields"]
    type: list
    elements: str
    version_added: '2.5.0'
  ipahomesrootdir:
    description: Default location of home directories.
    aliases: ["homesrootdir"]
    type: str
    version_added: '2.5.0'
  ipakrbauthzdata:
    description: Default types of PAC supported for services.
    aliases: ["krbauthzdata"]
    type: list
    elements: str
    choices: ["MS-PAC", "PAD", "nfs:NONE"]
    version_added: '2.5.0'
  ipamaxusernamelength:
    description: Maximum length of usernames.
    aliases: ["maxusernamelength"]
    type: int
    version_added: '2.5.0'
  ipapwdexpadvnotify:
    description: Notice of impending password expiration, in days.
    aliases: ["pwdexpadvnotify"]
    type: int
    version_added: '2.5.0'
  ipasearchrecordslimit:
    description: Maximum number of records to search (-1 or 0 is unlimited).
    aliases: ["searchrecordslimit"]
    type: int
    version_added: '2.5.0'
  ipasearchtimelimit:
    description: Maximum amount of time (seconds) for a search (-1 or 0 is unlimited).
    aliases: ["searchtimelimit"]
    type: int
    version_added: '2.5.0'
  ipaselinuxusermaporder:
    description: The SELinux user map order (order in increasing priority of SELinux users).
    aliases: ["selinuxusermaporder"]
    type: list
    elements: str
    version_added: '3.7.0'
  ipauserauthtype:
    description:
      - The authentication type to use by default.
      - The choice V(idp) has been added in community.general 7.3.0.
      - The choice V(passkey) has been added in community.general 8.1.0.
    aliases: ["userauthtype"]
    choices: ["password", "radius", "otp", "pkinit", "hardened", "idp", "passkey", "disabled"]
    type: list
    elements: str
    version_added: '2.5.0'
  ipauserobjectclasses:
    description: A list of user objectclasses.
    aliases: ["userobjectclasses"]
    type: list
    elements: str
    version_added: '7.3.0'
  ipausersearchfields:
    description: A list of fields to search in when searching for users.
    aliases: ["usersearchfields"]
    type: list
    elements: str
    version_added: '2.5.0'
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure password plugin features DC:Disable Last Success and KDC:Disable Lockout are enabled
  community.general.ipa_config:
    ipaconfigstring: ["KDC:Disable Last Success", "KDC:Disable Lockout"]
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the default login shell is bash
  community.general.ipa_config:
    ipadefaultloginshell: /bin/bash
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the default e-mail domain is ansible.com
  community.general.ipa_config:
    ipadefaultemaildomain: ansible.com
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the default primary group is set to ipausers
  community.general.ipa_config:
    ipadefaultprimarygroup: ipausers
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the group search fields are set to 'cn,description'
  community.general.ipa_config:
    ipagroupsearchfields: ['cn', 'description']
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the home directory location is set to /home
  community.general.ipa_config:
    ipahomesrootdir: /home
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the default types of PAC supported for services is set to MS-PAC and PAD
  community.general.ipa_config:
    ipakrbauthzdata: ["MS-PAC", "PAD"]
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the maximum user name length is set to 32
  community.general.ipa_config:
    ipamaxusernamelength: 32
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the password expiration notice is set to 4 days
  community.general.ipa_config:
    ipapwdexpadvnotify: 4
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the search record limit is set to 100
  community.general.ipa_config:
    ipasearchrecordslimit: 100
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the search time limit is set to 2 seconds
  community.general.ipa_config:
    ipasearchtimelimit: 2
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the default user auth type is password
  community.general.ipa_config:
    ipauserauthtype: ['password']
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the user search fields is set to 'uid,givenname,sn,ou,title'
  community.general.ipa_config:
    ipausersearchfields: ['uid', 'givenname', 'sn', 'ou', 'title']
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret

- name: Ensure the SELinux user map order is set
  community.general.ipa_config:
    ipaselinuxusermaporder:
      - "guest_u:s0"
      - "xguest_u:s0"
      - "user_u:s0"
      - "staff_u:s0-s0:c0.c1023"
      - "unconfined_u:s0-s0:c0.c1023"
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: supersecret
"""

RETURN = r"""
config:
  description: Configuration as returned by IPA API.
  returned: always
  type: dict
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec


class ConfigIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def config_show(self):
        return self._post_json(method="config_show", name=None)

    def config_mod(self, name, item):
        return self._post_json(method="config_mod", name=name, item=item)


def get_config_dict(
    ipaconfigstring=None,
    ipadefaultloginshell=None,
    ipadefaultemaildomain=None,
    ipadefaultprimarygroup=None,
    ipagroupsearchfields=None,
    ipagroupobjectclasses=None,
    ipahomesrootdir=None,
    ipakrbauthzdata=None,
    ipamaxusernamelength=None,
    ipapwdexpadvnotify=None,
    ipasearchrecordslimit=None,
    ipasearchtimelimit=None,
    ipaselinuxusermaporder=None,
    ipauserauthtype=None,
    ipausersearchfields=None,
    ipauserobjectclasses=None,
):
    config = {}
    if ipaconfigstring is not None:
        config["ipaconfigstring"] = ipaconfigstring
    if ipadefaultloginshell is not None:
        config["ipadefaultloginshell"] = ipadefaultloginshell
    if ipadefaultemaildomain is not None:
        config["ipadefaultemaildomain"] = ipadefaultemaildomain
    if ipadefaultprimarygroup is not None:
        config["ipadefaultprimarygroup"] = ipadefaultprimarygroup
    if ipagroupobjectclasses is not None:
        config["ipagroupobjectclasses"] = ipagroupobjectclasses
    if ipagroupsearchfields is not None:
        config["ipagroupsearchfields"] = ",".join(ipagroupsearchfields)
    if ipahomesrootdir is not None:
        config["ipahomesrootdir"] = ipahomesrootdir
    if ipakrbauthzdata is not None:
        config["ipakrbauthzdata"] = ipakrbauthzdata
    if ipamaxusernamelength is not None:
        config["ipamaxusernamelength"] = str(ipamaxusernamelength)
    if ipapwdexpadvnotify is not None:
        config["ipapwdexpadvnotify"] = str(ipapwdexpadvnotify)
    if ipasearchrecordslimit is not None:
        config["ipasearchrecordslimit"] = str(ipasearchrecordslimit)
    if ipasearchtimelimit is not None:
        config["ipasearchtimelimit"] = str(ipasearchtimelimit)
    if ipaselinuxusermaporder is not None:
        config["ipaselinuxusermaporder"] = "$".join(ipaselinuxusermaporder)
    if ipauserauthtype is not None:
        config["ipauserauthtype"] = ipauserauthtype
    if ipauserobjectclasses is not None:
        config["ipauserobjectclasses"] = ipauserobjectclasses
    if ipausersearchfields is not None:
        config["ipausersearchfields"] = ",".join(ipausersearchfields)

    return config


def get_config_diff(client, ipa_config, module_config):
    return client.get_diff(ipa_data=ipa_config, module_data=module_config)


def ensure(module, client):
    module_config = get_config_dict(
        ipaconfigstring=module.params.get("ipaconfigstring"),
        ipadefaultloginshell=module.params.get("ipadefaultloginshell"),
        ipadefaultemaildomain=module.params.get("ipadefaultemaildomain"),
        ipadefaultprimarygroup=module.params.get("ipadefaultprimarygroup"),
        ipagroupobjectclasses=module.params.get("ipagroupobjectclasses"),
        ipagroupsearchfields=module.params.get("ipagroupsearchfields"),
        ipahomesrootdir=module.params.get("ipahomesrootdir"),
        ipakrbauthzdata=module.params.get("ipakrbauthzdata"),
        ipamaxusernamelength=module.params.get("ipamaxusernamelength"),
        ipapwdexpadvnotify=module.params.get("ipapwdexpadvnotify"),
        ipasearchrecordslimit=module.params.get("ipasearchrecordslimit"),
        ipasearchtimelimit=module.params.get("ipasearchtimelimit"),
        ipaselinuxusermaporder=module.params.get("ipaselinuxusermaporder"),
        ipauserauthtype=module.params.get("ipauserauthtype"),
        ipausersearchfields=module.params.get("ipausersearchfields"),
        ipauserobjectclasses=module.params.get("ipauserobjectclasses"),
    )
    ipa_config = client.config_show()
    diff = get_config_diff(client, ipa_config, module_config)

    changed = False
    new_config = {}
    for module_key in diff:
        if module_config.get(module_key) != ipa_config.get(module_key, None):
            changed = True
            new_config.update({module_key: module_config.get(module_key)})

    if changed and not module.check_mode:
        client.config_mod(name=None, item=new_config)

    return changed, client.config_show()


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        ipaconfigstring=dict(
            type="list",
            elements="str",
            choices=[
                "AllowNThash",
                "KDC:Disable Last Success",
                "KDC:Disable Lockout",
                "KDC:Disable Default Preauth for SPNs",
            ],
            aliases=["configstring"],
        ),
        ipadefaultloginshell=dict(type="str", aliases=["loginshell"]),
        ipadefaultemaildomain=dict(type="str", aliases=["emaildomain"]),
        ipadefaultprimarygroup=dict(type="str", aliases=["primarygroup"]),
        ipagroupobjectclasses=dict(type="list", elements="str", aliases=["groupobjectclasses"]),
        ipagroupsearchfields=dict(type="list", elements="str", aliases=["groupsearchfields"]),
        ipahomesrootdir=dict(type="str", aliases=["homesrootdir"]),
        ipakrbauthzdata=dict(
            type="list", elements="str", choices=["MS-PAC", "PAD", "nfs:NONE"], aliases=["krbauthzdata"]
        ),
        ipamaxusernamelength=dict(type="int", aliases=["maxusernamelength"]),
        ipapwdexpadvnotify=dict(type="int", aliases=["pwdexpadvnotify"]),
        ipasearchrecordslimit=dict(type="int", aliases=["searchrecordslimit"]),
        ipasearchtimelimit=dict(type="int", aliases=["searchtimelimit"]),
        ipaselinuxusermaporder=dict(type="list", elements="str", aliases=["selinuxusermaporder"]),
        ipauserauthtype=dict(
            type="list",
            elements="str",
            aliases=["userauthtype"],
            choices=["password", "radius", "otp", "pkinit", "hardened", "idp", "passkey", "disabled"],
        ),
        ipausersearchfields=dict(type="list", elements="str", aliases=["usersearchfields"]),
        ipauserobjectclasses=dict(type="list", elements="str", aliases=["userobjectclasses"]),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    client = ConfigIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )

    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, user = ensure(module, client)
        module.exit_json(changed=changed, user=user)
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: x509_crl_info
version_added: '1.0.0'
short_description: Retrieve information on Certificate Revocation Lists (CRLs)
description:
  - This module allows one to retrieve information on Certificate Revocation Lists (CRLs).
author:
  - Felix Fontein (@felixfontein)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._attributes.info_module
  - community.crypto._attributes.idempotent_not_modify_state
  - community.crypto._cryptography_dep.minimum
  - community.crypto._name_encoding
options:
  path:
    description:
      - Remote absolute path where the generated CRL file should be created or is already located.
      - Either O(path) or O(content) must be specified, but not both.
    type: path
  content:
    description:
      - Content of the X.509 CRL in PEM format, or Base64-encoded X.509 CRL.
      - Either O(path) or O(content) must be specified, but not both.
    type: str
  list_revoked_certificates:
    description:
      - If set to V(false), the list of revoked certificates is not included in the result.
      - This is useful when retrieving information on large CRL files. Enumerating all revoked certificates can take some
        time, including serializing the result as JSON, sending it to the Ansible controller, and decoding it again.
    type: bool
    default: true
    version_added: 1.7.0

notes:
  - All timestamp values are provided in ASN.1 TIME format, in other words, following the C(YYYYMMDDHHMMSSZ) pattern. They
    are all in UTC.
seealso:
  - module: community.crypto.x509_crl
  - plugin: community.crypto.x509_crl_info
    plugin_type: filter
    description: A filter variant of this module.
  - plugin: community.crypto.to_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Get information on CRL
  community.crypto.x509_crl_info:
    path: /etc/ssl/my-ca.crl
  register: result

- name: Print the information
  ansible.builtin.debug:
    msg: "{{ result }}"

- name: Get information on CRL without list of revoked certificates
  community.crypto.x509_crl_info:
    path: /etc/ssl/very-large.crl
    list_revoked_certificates: false
  register: result
"""

RETURN = r"""
format:
  description:
    - Whether the CRL is in PEM format (V(pem)) or in DER format (V(der)).
  returned: success
  type: str
  sample: pem
  choices:
    - pem
    - der
issuer:
  description:
    - The CRL's issuer.
    - Note that for repeated values, only the last one will be returned.
    - See O(name_encoding) for how IDNs are handled.
  returned: success
  type: dict
  sample: {"organizationName": "Ansible", "commonName": "ca.example.com"}
issuer_ordered:
  description: The CRL's issuer as an ordered list of tuples.
  returned: success
  type: list
  elements: list
  sample: [["organizationName", "Ansible"], ["commonName": "ca.example.com"]]
last_update:
  description: The point in time from which this CRL can be trusted as ASN.1 TIME.
  returned: success
  type: str
  sample: '20190413202428Z'
next_update:
  description:
    - The point in time from which a new CRL will be issued and the client has to check for it as ASN.1 TIME.
    - Will be C(none) if no such timestamp is present.
  returned: success
  type: str
  sample: '20190413202428Z'
digest:
  description: The signature algorithm used to sign the CRL.
  returned: success
  type: str
  sample: sha256WithRSAEncryption
revoked_certificates:
  description: List of certificates to be revoked.
  returned: success if O(list_revoked_certificates=true)
  type: list
  elements: dict
  contains:
    serial_number:
      description:
        - Serial number of the certificate.
        - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string, such as C(11:22:33),
          you need to convert it to that form with P(community.crypto.to_serial#filter).
      type: int
      sample: 1234
    revocation_date:
      description: The point in time the certificate was revoked as ASN.1 TIME.
      type: str
      sample: '20190413202428Z'
    issuer:
      description:
        - The certificate's issuer.
        - See O(name_encoding) for how IDNs are handled.
      type: list
      elements: str
      sample: ["DNS:ca.example.org"]
    issuer_critical:
      description: Whether the certificate issuer extension is critical.
      type: bool
      sample: false
    reason:
      description:
        - The value for the revocation reason extension.
      type: str
      sample: key_compromise
      choices:
        - unspecified
        - key_compromise
        - ca_compromise
        - affiliation_changed
        - superseded
        - cessation_of_operation
        - certificate_hold
        - privilege_withdrawn
        - aa_compromise
        - remove_from_crl
    reason_critical:
      description: Whether the revocation reason extension is critical.
      type: bool
      sample: false
    invalidity_date:
      description: |-
        The point in time it was known/suspected that the private key was compromised
        or that the certificate otherwise became invalid as ASN.1 TIME.
      type: str
      sample: '20190413202428Z'
    invalidity_date_critical:
      description: Whether the invalidity date extension is critical.
      type: bool
      sample: false
"""


import base64
import binascii
import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.crl_info import (
    get_crl_info,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.pem import (
    identify_pem_format,
)


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "path": {"type": "path"},
            "content": {"type": "str"},
            "list_revoked_certificates": {"type": "bool", "default": True},
            "name_encoding": {
                "type": "str",
                "default": "ignore",
                "choices": ["ignore", "idna", "unicode"],
            },
        },
        required_one_of=(["path", "content"],),
        mutually_exclusive=(["path", "content"],),
        supports_check_mode=True,
    )

    content: str | None = module.params["content"]
    path: str | None = module.params["path"]
    if content is None:
        if path is None:
            module.fail_json(msg="One of content and path must be provided")
        try:
            with open(path, "rb") as f:
                data = f.read()
        except (IOError, OSError) as e:
            module.fail_json(msg=f"Error while reading CRL file from disk: {e}")
    else:
        data = content.encode("utf-8")
        if not identify_pem_format(data):
            try:
                data = base64.b64decode(content)
            except (binascii.Error, TypeError) as e:
                module.fail_json(msg=f"Error while Base64 decoding content: {e}")

    list_revoked_certificates: bool = module.params["list_revoked_certificates"]
    try:
        result = get_crl_info(
            module=module,
            content=data,
            list_revoked_certificates=list_revoked_certificates,
        )
        module.exit_json(**result)
    except OpenSSLObjectError as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2026 Klaas Weyermann (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_arcmachineextensions
version_added: "3.17.0"
short_description: Create, update and delete arc machine extensions.
description:
    - Create, update and delete arc machine extensions.
    - >-
      U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-hybridcompute/azure.mgmt.hybridcompute.operations.machineextensionsoperations?view=azure-python#azure-mgmt-hybridcompute-operations-machineextensionsoperations-begin-create-or-update)
options:
    name:
        description:
            - The name of the arc machine extension you're creating, updateing or deleting.
        required: true
        type: str
    machine_name:
        description:
            - The name of the arc machine.
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    location:
        description:
            - Location of the arc machine extension.
        required: false
        type: str
    properties:
        description:
            - Describes Machine Extension Properties.
            - Required for creation.
            - >-
              U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-hybridcompute/azure.mgmt.hybridcompute.models.machineextensionproperties?view=azure-python)
        type: dict
        suboptions:
            force_update_tag:
                description:
                    - How the extension handler should be forced to update even if the extension configuration has not changed.
                type: str
            publisher:
                description:
                    - The name of the extension handler publisher.
                type: str
            type:
                description:
                    - Specifies the type of the extension.
                type: str
            type_handler_version:
                description:
                    - Specifies the version of the script handler.
                type: str
            enable_automatic_upgrade:
                description:
                    - Indicates whether the extension should be automatically upgraded by the platform if there is a newer version available.
                type: bool
            auto_upgrade_minor_version:
                description:
                    - Indicates whether the extension should use a newer minor version if one is available at deployment time. Once deployed, however, the
                      extension will not upgrade minor versions unless redeployed, even with this property set to true.
                    - Does not work U(https://github.com/Azure/azure-rest-api-specs/issues/37591)
                type: bool
            settings:
                description:
                    - Public settings for the extension.
                type: dict
            protected_settings:
                description:
                    - Protected settings for the extension.
                type: dict
    state:
        description:
            - State of the arc machine extension
            - Use C(present) for creating/updating a arc machine extension.
            - Use C(absent) for deleting a arc machine extension.
        default: present
        type: str
        choices:
            - present
            - absent
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Klaas Weyermann (@Klaas-)
'''

EXAMPLES = '''
- name: Add a arc machine extension
  azure.azcollection.azure_rm_arcmachineextensions:
    state: present
    resource_group: resource_group_name
    machine_name: vm_name
    name: AzureMonitorLinuxAgent
    location: germanywestcentral
    properties:
      auto_upgrade_minor_version: false
      enable_automatic_upgrade: true
      publisher: Microsoft.Azure.Monitor
      type: AzureMonitorLinuxAgent
      type_handler_version: 1.37.0

- name: Delete a arc machine extension
  azure.azcollection.azure_rm_arcmachineextensions:
    state: absent
    name: arc_machine_extension_name
    machine_name: vm_name
    resource_group: resource_group_name
'''

RETURN = '''
arcmachineextension:
    description:
        - Details of the arc machine extension
        - Is null on state==absent (arc machine extension does not exist or will be deleted)
        - Assumes you make legal changes in check mode
    type: dict
    returned: always
    sample: {
        "id": \
"/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/ResourceGroup/providers/Microsoft.HybridCompute/machines/VMName/extensions/AzureMonitorLinuxAgent",
        "location": "germanywestcentral",
        "name": "AzureMonitorLinuxAgent",
        "properties":
        {
            "auto_upgrade_minor_version": false,
            "enable_automatic_upgrade": true,
            "force_update_tag": null,
            "instance_view":
            {
                "name": "AzureMonitorLinuxAgent",
                "status":
                {
                    "code": "0",
                    "level": "Information",
                    "message": "Extension Message: Enable succeeded",
                },
                "type": "AzureMonitorLinuxAgent",
                "type_handler_version": "1.37.0",
            },
            "protected_settings": null,
            "publisher": "Microsoft.Azure.Monitor",
            "settings": null,
            "type": "AzureMonitorLinuxAgent",
            "type_handler_version": "1.37.0",
        },
        "system_data":
        {
            "created_at": "2025-09-24T13:12:35.754905Z",
            "created_by": "your_sp",
            "created_by_type": "Application",
            "last_modified_at": "2025-09-24T13:12:35.754905Z",
            "last_modified_by": "your_sp",
            "last_modified_by_type": "Application",
        },
        "tags":
        {
            "Tag1": "Value1",
        },
        "type": "Microsoft.HybridCompute/machines/extensions",
    }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError

except ImportError:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'hybridcomputemachineextensions'

properties_spec = dict(
    force_update_tag=dict(type='str'),
    publisher=dict(type='str'),
    type=dict(type='str'),
    type_handler_version=dict(type='str'),
    enable_automatic_upgrade=dict(type='bool'),
    auto_upgrade_minor_version=dict(type='bool'),
    settings=dict(type='dict'),
    protected_settings=dict(type='dict'),
)


class AzureRMarcmachineextensions(AzureRMModuleBaseExt):
    """Information class for an Azure RM arc machine extensions"""

    def __init__(self):
        # https://learn.microsoft.com/en-us/python/api/azure-mgmt-hybridcompute/azure.mgmt.hybridcompute.models.machineextension?view=azure-python
        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            machine_name=dict(type='str', required=True),
            location=dict(type='str'),
            properties=dict(type='dict', options=properties_spec),
            state=dict(type='str', choices=['present', 'absent'], default='present')
        )

        self.name = None
        self.resource_group = None
        self.machine_name = None
        self.location = None
        self.tags = None
        self.properties = None
        self.state = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            arcmachineextension=dict(),
            diff=dict(
                before=None,
                after=None
            )
        )

        super(AzureRMarcmachineextensions, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        # Defaults for variables
        result = None
        result_compare = dict(compare=[])
        before_dict = None

        # Get current arc machine extension if it exists
        before_dict = self.get_arc_machine_extension()

        # Create dict from input, without None values
        # https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2018_03_01.models.arcmachineextensionresource?view=azure-python
        # tags seperately because of update_tags behavior
        arc_machine_extension_template = {
            "location": self.location,
            "name": self.name,
            "properties": self.properties
        }
        # Filter out all None values
        arc_machine_extension_input = {key: value for key, value in arc_machine_extension_template.items() if value is not None}

        # Create/Update if state==present
        if self.state == 'present':
            if before_dict is None:
                # arc machine extension does not exist, create
                # On creation input == what we send to api
                arc_machine_extension_update = arc_machine_extension_input
                # Needs to be extended by tags if set
                if self.tags:
                    arc_machine_extension_update['tags'] = self.tags
                # If no location is set default to the location of the resource group
                if not self.location:
                    resource_group = self.get_resource_group(self.resource_group)
                    arc_machine_extension_update['location'] = resource_group.location
                self.results['changed'] = True
                if self.check_mode:
                    # Check mode, skipping actual creation
                    pass
                else:
                    create_response = self.create_or_update(arc_machine_extension_update)
            else:
                # arc machine extension already exists, updating it
                # Dict for update is the union of existing object overwritten by input data
                arc_machine_extension_update = before_dict | arc_machine_extension_input

                # Enhanced with tags (special behaviour because of append_tags possibility)
                update_tags, update_tags_content = self.update_tags(before_dict.get('tags'))
                # Check if we need to update the arc machine extension
                if update_tags or not self.default_compare({}, arc_machine_extension_update, before_dict, '', result_compare):
                    arc_machine_extension_update['tags'] = update_tags_content
                    # Need to create/update the arc machine extension; changed -> True
                    self.results['changed'] = True
                    if self.check_mode:
                        # Check mode, skipping actual creation
                        pass
                    else:
                        create_response = self.create_or_update(arc_machine_extension_update)

            if self.check_mode or not self.results['changed']:
                # When object was not updated or when running in check mode
                # assume arc_machine_extension_update is resulting object
                result = arc_machine_extension_update
            else:
                # otherwise take resulting new object from response of create call
                result = create_response

        # Delete arc machine extension if state is absent and it exists
        # if it doesn't exist, it's already absent
        elif self.state == 'absent' and before_dict is not None:
            self.results['changed'] = True
            if self.check_mode:
                # do not delete in check mode
                pass
            else:
                self.delete()

        self.results['diff']['before'] = before_dict
        self.results['diff']['after'] = result
        self.results['arcmachineextension'] = result

        return self.results

    def get_arc_machine_extension(self):
        '''
        Gets the properties of the specified arc machine extension.

        :return: List of arc machine extensions
        '''
        self.log("Checking if arc machine extension {0} on machien {1} in resource group {2} is present".format(self.name,
                                                                                                                self.machine_name,
                                                                                                                self.resource_group))

        result = None
        response = None

        try:
            response = self.hybrid_compute_management_client.machine_extensions.get(extension_name=self.name,
                                                                                    machine_name=self.machine_name,
                                                                                    resource_group_name=self.resource_group)
        except ResourceNotFoundError as ex:
            self.log("Could not find arc machine extension {0} on machine {1} in resource group {2}".format(self.name, self.machine_name, self.resource_group))
        if response:
            result = self.serialize_obj(response, AZURE_OBJECT_CLASS)

        return result

    def create_or_update(self, arc_machine_extension_update):
        result = None
        response = None
        arc_machine_extensions = self.hybrid_compute_management_client.machine_extensions

        try:
            response = arc_machine_extensions.begin_create_or_update(resource_group_name=self.resource_group,
                                                                     machine_name=self.machine_name,
                                                                     extension_name=self.name,
                                                                     extension_parameters=arc_machine_extension_update,
                                                                     logging_enable=False)
        except Exception as ex:
            self.fail("Error creating or update arc machine extension {0} on machine {1} in resource group {2}: {3}".format(self.name,
                                                                                                                            self.machine_name,
                                                                                                                            self.resource_group,
                                                                                                                            str(ex)))

        if response:
            result = self.serialize_obj(self.get_poller_result(response), AZURE_OBJECT_CLASS)

        return result

    def delete(self):
        response = None
        try:
            response = self.hybrid_compute_management_client.machine_extensions.begin_delete(resource_group_name=self.resource_group,
                                                                                             machine_name=self.machine_name,
                                                                                             extension_name=self.name)
        except Exception as ex:
            self.fail("Error deleting arc machine {0} in resource group {1}: {2}".format(self.name, self.resource_group, str(ex)))

        if response:
            return self.get_poller_result(response)
        else:
            return None


def main():
    """Main execution"""
    AzureRMarcmachineextensions()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (C) 2019 Huawei
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

###############################################################################
# Documentation
###############################################################################

DOCUMENTATION = r"""
module: hwc_smn_topic
description:
  - Represents a SMN notification topic resource.
short_description: Creates a resource of SMNTopic in Huawei Cloud
author: Huawei Inc. (@huaweicloud)
requirements:
  - requests >= 2.18.4
  - keystoneauth1 >= 3.6.0
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - Whether the given object should exist in Huawei Cloud.
    type: str
    choices: ['present', 'absent']
    default: 'present'
  display_name:
    description:
      - Topic display name, which is presented as the name of the email sender in an email message. The topic display name
        contains a maximum of 192 bytes.
    type: str
  name:
    description:
      - Name of the topic to be created. The topic name is a string of 1 to 256 characters. It must contain upper- or lower-case
        letters, digits, hyphens (V(-)), and underscores (V(_)), and must start with a letter or digit.
    type: str
    required: true
extends_documentation_fragment:
  - community.general._hwc
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create a smn topic
  community.general.hwc_smn_topic:
    identity_endpoint: "{{ identity_endpoint }}"
    user_name: "{{ user_name }}"
    password: "{{ password }}"
    domain_name: "{{ domain_name }}"
    project_name: "{{ project_name }}"
    region: "{{ region }}"
    name: "ansible_smn_topic_test"
    state: present
"""

RETURN = r"""
create_time:
  description:
    - Time when the topic was created.
  returned: success
  type: str
display_name:
  description:
    - Topic display name, which is presented as the name of the email sender in an email message. The topic display name contains
      a maximum of 192 bytes.
  returned: success
  type: str
name:
  description:
    - Name of the topic to be created. The topic name is a string of 1 to 256 characters. It must contain upper- or lower-case
      letters, digits, hyphens (V(-)), and underscores (V(_)), and must start with a letter or digit.
  returned: success
  type: str
push_policy:
  description:
    - Message pushing policy. V(0) indicates that the message sending fails and the message is cached in the queue. V(1) indicates
      that the failed message is discarded.
  returned: success
  type: int
topic_urn:
  description:
    - Resource identifier of a topic, which is unique.
  returned: success
  type: str
update_time:
  description:
    - Time when the topic was updated.
  returned: success
  type: str
"""

###############################################################################
# Imports
###############################################################################

import re

from ansible_collections.community.general.plugins.module_utils._hwc_utils import (
    Config,
    HwcClientException,
    HwcModule,
    are_different_dicts,
    build_path,
    get_region,
    is_empty_value,
    navigate_value,
)

###############################################################################
# Main
###############################################################################


def main():
    """Main function"""

    module = HwcModule(
        argument_spec=dict(
            state=dict(default="present", choices=["present", "absent"], type="str"),
            display_name=dict(type="str"),
            name=dict(required=True, type="str"),
        ),
        supports_check_mode=True,
    )

    config = Config(module, "smn")

    state = module.params["state"]

    if not module.params.get("id"):
        module.params["id"] = get_resource_id(config)

    fetch = None
    link = self_link(module)
    # the link will include Nones if required format parameters are missed
    if not re.search("/None/|/None$", link):
        client = config.client(get_region(module), "smn", "project")
        fetch = fetch_resource(module, client, link)
    changed = False

    if fetch:
        if state == "present":
            expect = _get_resource_editable_properties(module)
            current_state = response_to_hash(module, fetch)
            current = {"display_name": current_state["display_name"]}
            if are_different_dicts(expect, current):
                if not module.check_mode:
                    fetch = update(config)
                    fetch = response_to_hash(module, fetch)
                changed = True
            else:
                fetch = current_state
        else:
            if not module.check_mode:
                delete(config)
                fetch = {}
            changed = True
    else:
        if state == "present":
            if not module.check_mode:
                fetch = create(config)
                fetch = response_to_hash(module, fetch)
            changed = True
        else:
            fetch = {}

    fetch.update({"changed": changed})

    module.exit_json(**fetch)


def create(config):
    module = config.module
    client = config.client(get_region(module), "smn", "project")

    link = "notifications/topics"
    r = None
    try:
        r = client.post(link, create_resource_opts(module))
    except HwcClientException as ex:
        msg = f"module(hwc_smn_topic): error creating resource, error: {ex}"
        module.fail_json(msg=msg)

    return get_resource(config, r)


def update(config):
    module = config.module
    client = config.client(get_region(module), "smn", "project")

    link = self_link(module)
    try:
        client.put(link, update_resource_opts(module))
    except HwcClientException as ex:
        msg = f"module(hwc_smn_topic): error updating resource, error: {ex}"
        module.fail_json(msg=msg)

    return fetch_resource(module, client, link)


def delete(config):
    module = config.module
    client = config.client(get_region(module), "smn", "project")

    link = self_link(module)
    try:
        client.delete(link)
    except HwcClientException as ex:
        msg = f"module(hwc_smn_topic): error deleting resource, error: {ex}"
        module.fail_json(msg=msg)


def fetch_resource(module, client, link):
    try:
        return client.get(link)
    except HwcClientException as ex:
        msg = f"module(hwc_smn_topic): error fetching resource, error: {ex}"
        module.fail_json(msg=msg)


def get_resource(config, result):
    module = config.module
    client = config.client(get_region(module), "smn", "project")

    v = ""
    try:
        v = navigate_value(result, ["topic_urn"])
    except Exception as ex:
        module.fail_json(msg=str(ex))

    d = {"topic_urn": v}
    url = build_path(module, "notifications/topics/{topic_urn}", d)

    return fetch_resource(module, client, url)


def get_resource_id(config):
    module = config.module
    client = config.client(get_region(module), "smn", "project")

    link = "notifications/topics"
    query_link = "?offset={offset}&limit=10"
    link += query_link

    p = {"offset": 0}
    v = module.params.get("name")
    ids = set()
    while True:
        r = None
        try:
            r = client.get(link.format(**p))
        except Exception:
            pass
        if r is None:
            break
        r = r.get("topics", [])
        if r == []:
            break
        for i in r:
            if i.get("name") == v:
                ids.add(i.get("topic_urn"))
        if len(ids) >= 2:
            module.fail_json(msg="Multiple resources are found")

        p["offset"] += 1

    return ids.pop() if ids else None


def self_link(module):
    return build_path(module, "notifications/topics/{id}")


def create_resource_opts(module):
    params = dict()

    v = module.params.get("display_name")
    if not is_empty_value(v):
        params["display_name"] = v

    v = module.params.get("name")
    if not is_empty_value(v):
        params["name"] = v

    return params


def update_resource_opts(module):
    params = dict()

    v = module.params.get("display_name")
    if not is_empty_value(v):
        params["display_name"] = v

    return params


def _get_resource_editable_properties(module):
    return {
        "display_name": module.params.get("display_name"),
    }


def response_to_hash(module, response):
    """Remove unnecessary properties from the response.
    This is for doing comparisons with Ansible's current parameters.
    """
    return {
        "create_time": response.get("create_time"),
        "display_name": response.get("display_name"),
        "name": response.get("name"),
        "push_policy": _push_policy_convert_from_response(response.get("push_policy")),
        "topic_urn": response.get("topic_urn"),
        "update_time": response.get("update_time"),
    }


def _push_policy_convert_from_response(value):
    return {
        0: "the message sending fails and is cached in the queue",
        1: "the failed message is discarded",
    }.get(int(value))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_alerts_profiles
short_description: Resource module for organizations _alerts _profiles
description:
  - Manage operations create, update and delete of the resource organizations _alerts
    _profiles.
  - Create an organization-wide alert configuration.
  - Removes an organization-wide alert config.
  - Update an organization-wide alert config.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  alertCondition:
    description: The conditions that determine if the alert triggers.
    suboptions:
      bit_rate_bps:
        description: The threshold the metric must cross to be valid for alerting.
          Used only for WAN Utilization alerts.
        type: int
      duration:
        description: The total duration in seconds that the threshold should be crossed
          before alerting.
        type: int
      interface:
        description: The uplink observed for the alert. Interface must be one of the
          following wan1, wan2, wan3, cellular, wan4.
        type: str
      jitter_ms:
        description: The threshold the metric must cross to be valid for alerting.
          Used only for VoIP Jitter alerts.
        type: int
      latency_ms:
        description: The threshold the metric must cross to be valid for alerting.
          Used only for WAN Latency alerts.
        type: int
      loss_ratio:
        description: The threshold the metric must cross to be valid for alerting.
          Used only for Packet Loss alerts.
        type: float
      mos:
        description: The threshold the metric must drop below to be valid for alerting.
          Used only for VoIP MOS alerts.
        type: float
      window:
        description: The look back period in seconds for sensing the alert.
        type: int
    type: dict
  alertConfigId:
    description: AlertConfigId path parameter. Alert config ID.
    type: str
  description:
    description: User supplied description of the alert.
    type: str
  enabled:
    description: Is the alert config enabled.
    type: bool
  networkTags:
    description: Networks with these tags will be monitored for the alert.
    elements: str
    type: list
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  recipients:
    description: List of recipients that will recieve the alert.
    suboptions:
      emails:
        description: A list of emails that will receive information about the alert.
        elements: str
        type: list
      httpServerIds:
        description: A list base64 encoded urls of webhook endpoints that will receive
          information about the alert.
        elements: str
        type: list
    type: dict
  type:
    description: The alert type.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationAlertsProfile
    description: Complete reference of the createOrganizationAlertsProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-alerts-profile
  - name: Cisco Meraki documentation for organizations deleteOrganizationAlertsProfile
    description: Complete reference of the deleteOrganizationAlertsProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-alerts-profile
  - name: Cisco Meraki documentation for organizations updateOrganizationAlertsProfile
    description: Complete reference of the updateOrganizationAlertsProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-alerts-profile
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_alerts_profile,
    organizations.Organizations.delete_organization_alerts_profile,
    organizations.Organizations.update_organization_alerts_profile,
  - Paths used are
    post /organizations/{organizationId}/alerts/profiles,
    delete /organizations/{organizationId}/alerts/profiles/{alertConfigId},
    put /organizations/{organizationId}/alerts/profiles/{alertConfigId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_alerts_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    alertCondition:
      bit_rate_bps: 10000
      duration: 60
      interface: wan1
      jitter_ms: 100
      latency_ms: 100
      loss_ratio: 0.1
      mos: 3.5
      window: 600
    description: WAN 1 high utilization
    networkTags:
      - tag1
      - tag2
    organizationId: string
    recipients:
      emails:
        - admin@example.org
      httpServerIds:
        - aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vcGF0aA==
    type: wanUtilization
- name: Update by id
  cisco.meraki.organizations_alerts_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    alertCondition:
      bit_rate_bps: 10000
      duration: 60
      interface: wan1
      jitter_ms: 100
      latency_ms: 100
      loss_ratio: 0.1
      mos: 3.5
      window: 600
    alertConfigId: string
    description: WAN 1 high utilization
    enabled: true
    networkTags:
      - tag1
      - tag2
    organizationId: string
    recipients:
      emails:
        - admin@example.org
      httpServerIds:
        - aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vcGF0aA==
    type: wanUtilization
- name: Delete by id
  cisco.meraki.organizations_alerts_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    alertConfigId: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "type": "string",
      "enabled": true,
      "alertCondition": {
        "duration": 0,
        "window": 0,
        "bit_rate_bps": 0,
        "interface": "string"
      },
      "recipients": {
        "emails": [
          "string"
        ],
        "httpServerIds": [
          "string"
        ]
      },
      "networkTags": [
        "string"
      ],
      "description": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Joshua Coronado (@joshuajcoronado) <joshua@coronado.io>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
author:
  - Joshua Coronado (@joshuajcoronado)
deprecated:
  alternative: cisco.meraki.networks_appliance_firewall_l7_firewall_rules
  removed_in: 3.0.0
  why: Updated modules released with increased functionality
description:
  - Allows for creation, management, and visibility into layer 7 firewalls implemented
    on Meraki MR access points.
  - Module assumes a complete list of firewall rules are passed as a parameter.
  - If there is interest in this module allowing manipulation of a single firewall
    rule, please submit an issue against this module.
extends_documentation_fragment: cisco.meraki.meraki
module: meraki_mr_l7_firewall
options:
  categories:
    description:
      - When C(True), specifies that applications and application categories should
        be queried instead of firewall rules.
    type: bool
  net_id:
    description:
      - ID of network containing access points.
    type: str
  net_name:
    description:
      - Name of network containing access points.
    type: str
  number:
    aliases:
      - ssid_number
    description:
      - Number of SSID to apply firewall rule to.
    type: str
  rules:
    description:
      - List of layer 7 firewall rules.
    elements: dict
    suboptions:
      application:
        description:
          - Application to filter.
        suboptions:
          id:
            description:
              - URI of application as defined by Meraki.
            type: str
          name:
            description:
              - Name of application to filter as defined by Meraki.
            type: str
        type: dict
      host:
        description:
          - FQDN of host to filter.
        type: str
      ip_range:
        description:
          - CIDR notation range of IP addresses to apply rule to.
          - Port can be appended to range with a C(":").
        type: str
      policy:
        choices:
          - deny
        default: deny
        description:
          - Policy to apply if rule is hit.
        type: str
      port:
        description:
          - TCP or UDP based port to filter.
        type: str
      type:
        choices:
          - application
          - application_category
          - host
          - ip_range
          - port
        description:
          - Type of policy to apply.
        type: str
    type: list
  ssid_name:
    aliases:
      - ssid
    description:
      - Name of SSID to apply firewall rule to.
    type: str
  state:
    choices:
      - present
      - query
    default: present
    description:
      - Query or modify a firewall rule.
    type: str
short_description: Manage MR access point layer 7 firewalls in the Meraki cloud
"""

EXAMPLES = r"""
- name: Query firewall rules
  meraki_mr_l7_firewall:
    auth_key: abc123
    org_name: YourOrg
    net_name: YourNet
    state: query
    number: 1
  delegate_to: localhost
- name: Query applications and application categories
  meraki_mr_l7_firewall:
    auth_key: abc123
    org_name: YourOrg
    net_name: YourNet
    number: 1
    categories: true
    state: query
  delegate_to: localhost
- name: Set firewall rules
  meraki_mr_l7_firewall:
    auth_key: abc123
    org_name: YourOrg
    net_name: YourNet
    number: 1
    state: present
    rules:
      - policy: deny
        type: port
        port: 8080
      - type: port
        port: 1234
      - type: host
        host: asdf.com
      - type: application
        application:
          id: meraki:layer7/application/205
      - type: application_category
        application:
          id: meraki:layer7/category/24
  delegate_to: localhost
"""

RETURN = r"""
data:
    description: Firewall rules associated to network SSID.
    returned: success
    type: complex
    contains:
        rules:
            description: Ordered list of firewall rules.
            returned: success, when not querying applications
            type: list
            contains:
                policy:
                    description: Action to apply when rule is hit.
                    returned: success
                    type: str
                    sample: deny
                type:
                    description: Type of rule category.
                    returned: success
                    type: str
                    sample: applications
                applications:
                    description: List of applications within a category.
                    type: list
                    contains:
                        id:
                            description: URI of application.
                            returned: success
                            type: str
                            sample: Gmail
                        name:
                            description: Descriptive name of application.
                            returned: success
                            type: str
                            sample: meraki:layer7/application/4
                applicationCategory:
                    description: List of application categories within a category.
                    type: list
                    contains:
                        id:
                            description: URI of application.
                            returned: success
                            type: str
                            sample: Gmail
                        name:
                            description: Descriptive name of application.
                            returned: success
                            type: str
                            sample: meraki:layer7/application/4
                port:
                    description: Port number in rule.
                    returned: success
                    type: str
                    sample: 23
                ipRange:
                    description: Range of IP addresses in rule.
                    returned: success
                    type: str
                    sample: 1.1.1.0/23
        application_categories:
            description: List of application categories and applications.
            type: list
            returned: success, when querying applications
            contains:
                applications:
                    description: List of applications within a category.
                    type: list
                    contains:
                        id:
                            description: URI of application.
                            returned: success
                            type: str
                            sample: Gmail
                        name:
                            description: Descriptive name of application.
                            returned: success
                            type: str
                            sample: meraki:layer7/application/4
                id:
                    description: URI of application category.
                    returned: success
                    type: str
                    sample: Email
                name:
                    description: Descriptive name of application category.
                    returned: success
                    type: str
                    sample: layer7/category/1
"""

import copy
from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)


def get_applications(meraki, net_id):
    path = meraki.construct_path("get_categories", net_id=net_id)
    return meraki.request(path, method="GET")


def lookup_application(meraki, net_id, application):
    response = get_applications(meraki, net_id)
    for category in response["applicationCategories"]:
        if category["name"].lower() == application.lower():
            return category["id"]
        for app in category["applications"]:
            if app["name"].lower() == application.lower():
                return app["id"]
    meraki.fail_json(
        msg="No application or category named {0} found".format(application)
    )


def assemble_payload(meraki, net_id, rule):
    new_rule = {}
    if rule["type"] == "application":
        new_rule = {
            "policy": rule["policy"],
            "type": "application",
        }
        if rule["application"]["id"]:
            new_rule["value"] = {"id": rule["application"]["id"]}
        elif rule["application"]["name"]:
            new_rule["value"] = {
                "id": lookup_application(meraki, net_id, rule["application"]["name"])
            }
    elif rule["type"] == "application_category":
        new_rule = {
            "policy": rule["policy"],
            "type": "applicationCategory",
        }
        if rule["application"]["id"]:
            new_rule["value"] = {"id": rule["application"]["id"]}
        elif rule["application"]["name"]:
            new_rule["value"] = {
                "id": lookup_application(meraki, net_id, rule["application"]["name"])
            }
    elif rule["type"] == "ip_range":
        new_rule = {
            "policy": rule["policy"],
            "type": "ipRange",
            "value": rule["ip_range"],
        }
    elif rule["type"] == "host":
        new_rule = {
            "policy": rule["policy"],
            "type": rule["type"],
            "value": rule["host"],
        }
    elif rule["type"] == "port":
        new_rule = {
            "policy": rule["policy"],
            "type": rule["type"],
            "value": rule["port"],
        }
    return new_rule


def restructure_response(rules):
    for rule in rules["rules"]:
        type = rule["type"]
        rule[type] = copy.deepcopy(rule["value"])
        del rule["value"]
    return rules


def get_ssid_number(name, data):
    for ssid in data:
        if name == ssid["name"]:
            return ssid["number"]
    return False


def get_ssids(meraki, net_id):
    path = meraki.construct_path("get_ssids", net_id=net_id)
    return meraki.request(path, method="GET")


def get_rules(meraki, net_id, number):
    path = meraki.construct_path("get_all", net_id=net_id, custom={"number": number})
    response = meraki.request(path, method="GET")
    if meraki.status == 200:
        return response


def main():
    # define the available arguments/parameters that a user can pass to
    # the module

    application_arg_spec = dict(
        id=dict(type="str"),
        name=dict(type="str"),
    )

    rule_arg_spec = dict(
        policy=dict(type="str", choices=["deny"], default="deny"),
        type=dict(
            type="str",
            choices=["application", "application_category", "host", "ip_range", "port"],
        ),
        ip_range=dict(type="str"),
        application=dict(type="dict", default=None, options=application_arg_spec),
        host=dict(type="str"),
        port=dict(type="str"),
    )

    argument_spec = meraki_argument_spec()
    argument_spec.update(
        state=dict(type="str", choices=["present", "query"], default="present"),
        net_name=dict(type="str"),
        net_id=dict(type="str"),
        number=dict(type="str", aliases=["ssid_number"]),
        ssid_name=dict(type="str", aliases=["ssid"]),
        rules=dict(type="list", default=None, elements="dict", options=rule_arg_spec),
        categories=dict(type="bool"),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    meraki = MerakiModule(module, function="mr_l7_firewall")

    # check for argument completeness
    if meraki.params["rules"]:
        for rule in meraki.params["rules"]:
            if rule["type"] == "application" and rule["application"] is None:
                meraki.fail_json(
                    msg="application argument is required when type is application."
                )
            elif rule["type"] == "application_category" and rule["application"] is None:
                meraki.fail_json(
                    msg="application argument is required when type is application_category."
                )
            elif rule["type"] == "host" and rule["host"] is None:
                meraki.fail_json(msg="host argument is required when type is host.")
            elif rule["type"] == "port" and rule["port"] is None:
                meraki.fail_json(msg="port argument is required when type is port.")

    meraki.params["follow_redirects"] = "all"
    query_ssids_urls = {"mr_l7_firewall": "/networks/{net_id}/wireless/ssids"}
    query_urls = {
        "mr_l7_firewall": "/networks/{net_id}/wireless/ssids/{number}/firewall/l7FirewallRules"
    }
    update_urls = {
        "mr_l7_firewall": "/networks/{net_id}/wireless/ssids/{number}/firewall/l7FirewallRules"
    }
    query_category_urls = {
        "mr_l7_firewall": "/networks/{net_id}/trafficShaping/applicationCategories"
    }

    meraki.url_catalog["get_all"].update(query_urls)
    meraki.url_catalog["get_categories"] = query_category_urls
    meraki.url_catalog["get_ssids"] = query_ssids_urls
    meraki.url_catalog["update"] = update_urls

    payload = None

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)
    org_id = meraki.params["org_id"]
    orgs = None
    if org_id is None:
        orgs = meraki.get_orgs()
        for org in orgs:
            if org["name"] == meraki.params["org_name"]:
                org_id = org["id"]
    net_id = meraki.params["net_id"]
    if net_id is None:
        if orgs is None:
            orgs = meraki.get_orgs()
        net_id = meraki.get_net_id(
            net_name=meraki.params["net_name"], data=meraki.get_nets(org_id=org_id)
        )
    number = meraki.params["number"]
    if meraki.params["ssid_name"]:
        ssids = get_ssids(meraki, net_id)
        number = get_ssid_number(meraki.params["ssid_name"], ssids)

    if meraki.params["state"] == "query":
        if meraki.params["categories"] is True:  # Output only applications
            meraki.result["data"] = get_applications(meraki, net_id)
        else:
            meraki.result["data"] = restructure_response(
                get_rules(meraki, net_id, number)
            )
    elif meraki.params["state"] == "present":
        rules = get_rules(meraki, net_id, number)
        path = meraki.construct_path(
            "get_all", net_id=net_id, custom={"number": number}
        )

        # Detect if no rules are given, special case
        if len(meraki.params["rules"]) == 0:
            # Conditionally wrap parameters in rules makes it comparable
            if isinstance(meraki.params["rules"], list):
                param_rules = {"rules": meraki.params["rules"]}
            else:
                param_rules = meraki.params["rules"]
            if meraki.is_update_required(rules, param_rules):
                if meraki.module.check_mode is True:
                    meraki.result["data"] = meraki.params["rules"]
                    meraki.result["changed"] = True
                    meraki.exit_json(**meraki.result)
                payload = {"rules": []}
                response = meraki.request(
                    path, method="PUT", payload=json.dumps(payload)
                )
                meraki.result["data"] = response
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
            else:
                meraki.result["data"] = param_rules
                meraki.exit_json(**meraki.result)
        if meraki.params["rules"]:
            payload = {"rules": []}
            for rule in meraki.params["rules"]:
                payload["rules"].append(assemble_payload(meraki, net_id, rule))
        else:
            payload = dict()
        if meraki.is_update_required(rules, payload, force_include="id"):
            if meraki.module.check_mode is True:
                response = restructure_response(payload)
                meraki.generate_diff(restructure_response(rules), response)
                meraki.result["data"] = response
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
            response = meraki.request(path, method="PUT", payload=json.dumps(payload))
            response = restructure_response(response)
            if meraki.status == 200:
                meraki.generate_diff(restructure_response(rules), response)
                meraki.result["data"] = response
                meraki.result["changed"] = True
        else:
            if meraki.module.check_mode is True:
                meraki.result["data"] = rules
                meraki.result["changed"] = False
                meraki.exit_json(**meraki.result)
            meraki.result["data"] = payload

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# Copyright (c) 2023, Léo El Amri (@lel-amri)
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_compose_v2

short_description: Manage multi-container Docker applications with Docker Compose CLI plugin

version_added: 3.6.0

description:
  - Uses Docker Compose to start or shutdown services.
extends_documentation_fragment:
  - community.docker._compose_v2
  - community.docker._compose_v2.minimum_version
  - community.docker._docker.cli_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
    details:
      - In check mode, pulling the image does not result in a changed result.
  diff_mode:
    support: none
  idempotent:
    support: partial
    details:
      - If O(state=restarted) or O(recreate=always) the module is not idempotent.

options:
  state:
    description:
      - Desired state of the project.
      - V(present) is equivalent to running C(docker compose up).
      - V(stopped) is equivalent to running C(docker compose stop).
      - V(absent) is equivalent to running C(docker compose down).
      - V(restarted) is equivalent to running C(docker compose restart).
    type: str
    default: present
    choices:
      - absent
      - stopped
      - restarted
      - present
  pull:
    description:
      - Whether to pull images before running. This is used when C(docker compose up) is run.
      - V(always) ensures that the images are always pulled, even when already present on the Docker daemon.
      - V(missing) only pulls them when they are not present on the Docker daemon.
      - V(never) never pulls images. If they are not present, the module will fail when trying to create the containers that
        need them.
      - V(policy) use the Compose file's C(pull_policy) defined for the service to figure out what to do.
    type: str
    choices:
      - always
      - missing
      - never
      - policy
    default: policy
  build:
    description:
      - Whether to build images before starting containers. This is used when C(docker compose up) is run.
      - V(always) always builds before starting containers. This is equivalent to the C(--build) option of C(docker compose
        up).
      - V(never) never builds before starting containers. This is equivalent to the C(--no-build) option of C(docker compose
        up).
      - V(policy) uses the policy as defined in the Compose file.
    type: str
    choices:
      - always
      - never
      - policy
    default: policy
  dependencies:
    description:
      - When O(state) is V(present) or V(restarted), specify whether or not to include linked services.
    type: bool
    default: true
  ignore_build_events:
    description:
      - Ignores image building events for change detection.
      - If O(state=present) and O(ignore_build_events=true) and O(build=always), a rebuild that does not trigger a container
        restart no longer results in RV(ignore:changed=true).
      - Note that Docker Compose 2.31.0 is the first Compose 2.x version to emit build events. For older versions, the behavior
        is always as if O(ignore_build_events=true).
    type: bool
    default: true
    version_added: 4.2.0
  recreate:
    description:
      - By default containers will be recreated when their configuration differs from the service definition.
      - Setting to V(never) ignores configuration differences and leaves existing containers unchanged.
      - Setting to V(always) forces recreation of all existing containers.
    type: str
    default: auto
    choices:
      - always
      - never
      - auto
  renew_anon_volumes:
    description:
      - Whether to recreate instead of reuse anonymous volumes from previous containers.
      - V(true) is equivalent to the C(--renew-anon-volumes) option of C(docker compose up).
    type: bool
    default: false
    version_added: 4.0.0
  remove_images:
    description:
      - Use with O(state=absent) to remove all images or only local images.
    type: str
    choices:
      - all
      - local
  remove_volumes:
    description:
      - Use with O(state=absent) to remove data volumes.
    type: bool
    default: false
  remove_orphans:
    description:
      - Remove containers for services not defined in the Compose file.
    type: bool
    default: false
  timeout:
    description:
      - Timeout in seconds for container shutdown when attached or when containers are already running.
    type: int
  services:
    description:
      - Specifies a subset of services to be targeted.
    type: list
    elements: str
  scale:
    description:
      - Define how to scale services when running C(docker compose up).
      - Provide a dictionary of key/value pairs where the key is the name of the service and the value is an integer count
        for the number of containers.
    type: dict
    version_added: 3.7.0
  wait:
    description:
      - When running C(docker compose up), pass C(--wait) to wait for services to be running/healthy.
      - A timeout can be set with the O(wait_timeout) option.
    type: bool
    default: false
    version_added: 3.8.0
  wait_timeout:
    description:
      - When O(wait=true), wait at most this amount of seconds.
    type: int
    version_added: 3.8.0
  assume_yes:
    description:
      - When O(assume_yes=true), pass C(-y)/C(--yes) to assume "yes" as answer to all prompts and run non-interactively.
      - Right now a prompt is asked whenever a non-matching volume should be re-created. O(assume_yes=false)
        results in the question being answered by "no", which will simply re-use the existing volume.
      - This option is only available on Docker Compose 2.32.0 or newer.
    type: bool
    default: false
    version_added: 4.5.0

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_compose_v2_pull
"""

EXAMPLES = r"""
---
# Examples use the django example at https://docs.docker.com/compose/django. Follow it to create the
# flask directory

- name: Run using a project directory
  hosts: localhost
  gather_facts: false
  tasks:
    - name: Tear down existing services
      community.docker.docker_compose_v2:
        project_src: flask
        state: absent

    - name: Create and start services
      community.docker.docker_compose_v2:
        project_src: flask
      register: output

    - name: Show results
      ansible.builtin.debug:
        var: output

    - name: Run `docker compose up` again
      community.docker.docker_compose_v2:
        project_src: flask
      register: output

    - name: Show results
      ansible.builtin.debug:
        var: output

    - ansible.builtin.assert:
        that: not output.changed

    - name: Stop all services
      community.docker.docker_compose_v2:
        project_src: flask
        state: stopped
      register: output

    - name: Show results
      ansible.builtin.debug:
        var: output

    - name: Verify that web and db services are not running
      ansible.builtin.assert:
        that:
          - web_container.State != 'running'
          - db_container.State != 'running'
      vars:
        web_container: >-
          {{ output.containers | selectattr("Service", "equalto", "web") | first }}
        db_container: >-
          {{ output.containers | selectattr("Service", "equalto", "db") | first }}

    - name: Restart services
      community.docker.docker_compose_v2:
        project_src: flask
        state: restarted
      register: output

    - name: Show results
      ansible.builtin.debug:
        var: output

    - name: Verify that web and db services are running
      ansible.builtin.assert:
        that:
          - web_container.State == 'running'
          - db_container.State == 'running'
      vars:
        web_container: >-
          {{ output.containers | selectattr("Service", "equalto", "web") | first }}
        db_container: >-
          {{ output.containers | selectattr("Service", "equalto", "db") | first }}
"""

RETURN = r"""
containers:
  description:
    - A list of containers associated to the service.
  returned: success
  type: list
  elements: dict
  contains:
    Command:
      description:
        - The container's command.
      type: raw
    CreatedAt:
      description:
        - The timestamp when the container was created.
      type: str
      sample: "2024-01-02 12:20:41 +0100 CET"
    ExitCode:
      description:
        - The container's exit code.
      type: int
    Health:
      description:
        - The container's health check.
      type: raw
    ID:
      description:
        - The container's ID.
      type: str
      sample: "44a7d607219a60b7db0a4817fb3205dce46e91df2cb4b78a6100b6e27b0d3135"
    Image:
      description:
        - The container's image.
      type: str
    Labels:
      description:
        - Labels for this container.
      type: dict
    LocalVolumes:
      description:
        - The local volumes count.
      type: str
    Mounts:
      description:
        - Mounts.
      type: str
    Name:
      description:
        - The container's primary name.
      type: str
    Names:
      description:
        - List of names of the container.
      type: list
      elements: str
    Networks:
      description:
        - List of networks attached to this container.
      type: list
      elements: str
    Ports:
      description:
        - List of port assignments as a string.
      type: str
    Publishers:
      description:
        - List of port assigments.
      type: list
      elements: dict
      contains:
        URL:
          description:
            - Interface the port is bound to.
          type: str
        TargetPort:
          description:
            - The container's port the published port maps to.
          type: int
        PublishedPort:
          description:
            - The port that is published.
          type: int
        Protocol:
          description:
            - The protocol.
          type: str
          choices:
            - tcp
            - udp
    RunningFor:
      description:
        - Amount of time the container runs.
      type: str
    Service:
      description:
        - The name of the service.
      type: str
    Size:
      description:
        - The container's size.
      type: str
      sample: "0B"
    State:
      description:
        - The container's state.
      type: str
      sample: running
    Status:
      description:
        - The container's status.
      type: str
      sample: Up About a minute
images:
  description:
    - A list of images associated to the service.
  returned: success
  type: list
  elements: dict
  contains:
    ID:
      description:
        - The image's ID.
      type: str
      sample: sha256:c8bccc0af9571ec0d006a43acb5a8d08c4ce42b6cc7194dd6eb167976f501ef1
    ContainerName:
      description:
        - Name of the conainer this image is used by.
      type: str
    Repository:
      description:
        - The repository where this image belongs to.
      type: str
    Tag:
      description:
        - The tag of the image.
      type: str
    Size:
      description:
        - The image's size in bytes.
      type: int
actions:
  description:
    - A list of actions that have been applied.
  returned: success
  type: list
  elements: dict
  contains:
    what:
      description:
        - What kind of resource was changed.
      type: str
      sample: container
      choices:
        - container
        - image
        - network
        - service
        - unknown
        - volume
    id:
      description:
        - The ID of the resource that was changed.
      type: str
      sample: container
    status:
      description:
        - The status change that happened.
      type: str
      sample: Creating
      choices:
        - Starting
        - Exiting
        - Restarting
        - Creating
        - Stopping
        - Killing
        - Removing
        - Recreating
        - Pulling
        - Building
"""

import traceback
import typing as t

from ansible.module_utils.common.validation import check_type_int

from ansible_collections.community.docker.plugins.module_utils._common_cli import (
    AnsibleModuleDockerClient,
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._compose_v2 import (
    BaseComposeManager,
    common_compose_argspec_ex,
    is_failed,
)
from ansible_collections.community.docker.plugins.module_utils._version import (
    LooseVersion,
)


class ServicesManager(BaseComposeManager):
    def __init__(self, client: AnsibleModuleDockerClient) -> None:
        super().__init__(client)
        parameters = self.client.module.params

        self.state: t.Literal["absent", "present", "stopped", "restarted"] = parameters[
            "state"
        ]
        self.dependencies: bool = parameters["dependencies"]
        self.pull: t.Literal["always", "missing", "never", "policy"] = parameters[
            "pull"
        ]
        self.build: t.Literal["always", "never", "policy"] = parameters["build"]
        self.ignore_build_events: bool = parameters["ignore_build_events"]
        self.recreate: t.Literal["always", "never", "auto"] = parameters["recreate"]
        self.remove_images: t.Literal["all", "local"] | None = parameters[
            "remove_images"
        ]
        self.remove_volumes: bool = parameters["remove_volumes"]
        self.remove_orphans: bool = parameters["remove_orphans"]
        self.renew_anon_volumes: bool = parameters["renew_anon_volumes"]
        self.timeout: int | None = parameters["timeout"]
        self.services: list[str] = parameters["services"] or []
        self.scale: dict[str, t.Any] = parameters["scale"] or {}
        self.wait: bool = parameters["wait"]
        self.wait_timeout: int | None = parameters["wait_timeout"]
        self.yes: bool = parameters["assume_yes"]
        if self.compose_version < LooseVersion("2.32.0") and self.yes:
            self.fail(
                f"assume_yes=true needs Docker Compose 2.32.0 or newer, not version {self.compose_version}"
            )

        for key, value in self.scale.items():
            if not isinstance(key, str):
                self.fail(f"The key {key!r} for `scale` is not a string")
            try:
                value = check_type_int(value)
            except TypeError:
                self.fail(f"The value {value!r} for `scale[{key!r}]` is not an integer")
            if value < 0:
                self.fail(f"The value {value!r} for `scale[{key!r}]` is negative")
            self.scale[key] = value

    def run(self) -> dict[str, t.Any]:
        if self.state == "present":
            result = self.cmd_up()
        elif self.state == "stopped":
            result = self.cmd_stop()
        elif self.state == "restarted":
            result = self.cmd_restart()
        elif self.state == "absent":
            result = self.cmd_down()
        else:
            raise AssertionError("Unexpected state")

        result["containers"] = self.list_containers()
        result["images"] = self.list_images()
        self.cleanup_result(result)
        return result

    def get_up_cmd(self, dry_run: bool, no_start: bool = False) -> list[str]:
        args = self.get_base_args() + ["up", "--detach", "--no-color", "--quiet-pull"]
        if self.pull != "policy":
            args.extend(["--pull", self.pull])
        if self.remove_orphans:
            args.append("--remove-orphans")
        if self.recreate == "always":
            args.append("--force-recreate")
        if self.recreate == "never":
            args.append("--no-recreate")
        if self.renew_anon_volumes:
            args.append("--renew-anon-volumes")
        if not self.dependencies:
            args.append("--no-deps")
        if self.timeout is not None:
            args.extend(["--timeout", f"{self.timeout}"])
        if self.build == "always":
            args.append("--build")
        elif self.build == "never":
            args.append("--no-build")
        for key, value in sorted(self.scale.items()):
            args.extend(["--scale", f"{key}={value}"])
        if self.wait:
            args.append("--wait")
            if self.wait_timeout is not None:
                args.extend(["--wait-timeout", str(self.wait_timeout)])
        if no_start:
            args.append("--no-start")
        if dry_run:
            args.append("--dry-run")
        if self.yes:
            # Note that for Docker Compose 2.32.x and 2.33.x, the long form is '--y' and not '--yes'.
            # This was fixed in Docker Compose 2.34.0 (https://github.com/docker/compose/releases/tag/v2.34.0).
            args.append(
                "-y" if self.compose_version < LooseVersion("2.34.0") else "--yes"
            )
        args.append("--")
        for service in self.services:
            args.append(service)
        return args

    def cmd_up(self) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {}
        args = self.get_up_cmd(self.check_mode)
        rc, stdout, stderr = self.client.call_cli(*args, cwd=self.project_src)
        events = self.parse_events(stderr, dry_run=self.check_mode, nonzero_rc=rc != 0)
        self.emit_warnings(events)
        self.update_result(
            result,
            events,
            stdout,
            stderr,
            ignore_service_pull_events=True,
            ignore_build_events=self.ignore_build_events,
        )
        self.update_failed(result, events, args, stdout, stderr, rc)
        return result

    def get_stop_cmd(self, dry_run: bool) -> list[str]:
        args = self.get_base_args() + ["stop"]
        if self.timeout is not None:
            args.extend(["--timeout", f"{self.timeout}"])
        if dry_run:
            args.append("--dry-run")
        args.append("--")
        for service in self.services:
            args.append(service)
        return args

    def _are_containers_stopped(self) -> bool:
        return all(
            container["State"] in ("created", "exited", "stopped", "killed")
            for container in self.list_containers_raw()
        )

    def cmd_stop(self) -> dict[str, t.Any]:
        # Since 'docker compose stop' **always** claims it is stopping containers, even if they are already
        # stopped, we have to do this a bit more complicated.

        result: dict[str, t.Any] = {}
        # Make sure all containers are created
        args_1 = self.get_up_cmd(self.check_mode, no_start=True)
        rc_1, stdout_1, stderr_1 = self.client.call_cli(*args_1, cwd=self.project_src)
        events_1 = self.parse_events(
            stderr_1, dry_run=self.check_mode, nonzero_rc=rc_1 != 0
        )
        self.emit_warnings(events_1)
        self.update_result(
            result,
            events_1,
            stdout_1,
            stderr_1,
            ignore_service_pull_events=True,
            ignore_build_events=self.ignore_build_events,
        )
        is_failed_1 = is_failed(events_1, rc_1)
        if not is_failed_1 and not self._are_containers_stopped():
            # Make sure all containers are stopped
            args_2 = self.get_stop_cmd(self.check_mode)
            rc_2, stdout_2, stderr_2 = self.client.call_cli(
                *args_2, cwd=self.project_src
            )
            events_2 = self.parse_events(
                stderr_2, dry_run=self.check_mode, nonzero_rc=rc_2 != 0
            )
            self.emit_warnings(events_2)
            self.update_result(result, events_2, stdout_2, stderr_2)
        else:
            args_2 = []
            rc_2, stdout_2, stderr_2 = 0, b"", b""
            events_2 = []
        # Compose result
        self.update_failed(
            result,
            events_1 + events_2,
            args_1 if is_failed_1 else args_2,
            stdout_1 if is_failed_1 else stdout_2,
            stderr_1 if is_failed_1 else stderr_2,
            rc_1 if is_failed_1 else rc_2,
        )
        return result

    def get_restart_cmd(self, dry_run: bool) -> list[str]:
        args = self.get_base_args() + ["restart"]
        if not self.dependencies:
            args.append("--no-deps")
        if self.timeout is not None:
            args.extend(["--timeout", f"{self.timeout}"])
        if dry_run:
            args.append("--dry-run")
        args.append("--")
        for service in self.services:
            args.append(service)
        return args

    def cmd_restart(self) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {}
        args = self.get_restart_cmd(self.check_mode)
        rc, stdout, stderr = self.client.call_cli(*args, cwd=self.project_src)
        events = self.parse_events(stderr, dry_run=self.check_mode, nonzero_rc=rc != 0)
        self.emit_warnings(events)
        self.update_result(result, events, stdout, stderr)
        self.update_failed(result, events, args, stdout, stderr, rc)
        return result

    def get_down_cmd(self, dry_run: bool) -> list[str]:
        args = self.get_base_args() + ["down"]
        if self.remove_orphans:
            args.append("--remove-orphans")
        if self.remove_images:
            args.extend(["--rmi", self.remove_images])
        if self.remove_volumes:
            args.append("--volumes")
        if self.timeout is not None:
            args.extend(["--timeout", f"{self.timeout}"])
        if dry_run:
            args.append("--dry-run")
        args.append("--")
        for service in self.services:
            args.append(service)
        return args

    def cmd_down(self) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {}
        args = self.get_down_cmd(self.check_mode)
        rc, stdout, stderr = self.client.call_cli(*args, cwd=self.project_src)
        events = self.parse_events(stderr, dry_run=self.check_mode, nonzero_rc=rc != 0)
        self.emit_warnings(events)
        self.update_result(result, events, stdout, stderr)
        self.update_failed(result, events, args, stdout, stderr, rc)
        return result


def main() -> None:
    argument_spec = {
        "state": {
            "type": "str",
            "default": "present",
            "choices": ["absent", "present", "stopped", "restarted"],
        },
        "dependencies": {"type": "bool", "default": True},
        "pull": {
            "type": "str",
            "choices": ["always", "missing", "never", "policy"],
            "default": "policy",
        },
        "build": {
            "type": "str",
            "choices": ["always", "never", "policy"],
            "default": "policy",
        },
        "recreate": {
            "type": "str",
            "default": "auto",
            "choices": ["always", "never", "auto"],
        },
        "renew_anon_volumes": {"type": "bool", "default": False},
        "remove_images": {"type": "str", "choices": ["all", "local"]},
        "remove_volumes": {"type": "bool", "default": False},
        "remove_orphans": {"type": "bool", "default": False},
        "timeout": {"type": "int"},
        "services": {"type": "list", "elements": "str"},
        "scale": {"type": "dict"},
        "wait": {"type": "bool", "default": False},
        "wait_timeout": {"type": "int"},
        "ignore_build_events": {"type": "bool", "default": True},
        "assume_yes": {"type": "bool", "default": False},
    }
    argspec_ex = common_compose_argspec_ex()
    argument_spec.update(argspec_ex.pop("argspec"))

    client = AnsibleModuleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        needs_api_version=False,
        **argspec_ex,
    )

    try:
        manager = ServicesManager(client)
        result = manager.run()
        manager.cleanup()
        client.module.exit_json(**result)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>
# Copyright: (c) 2024, Noppanut Ploywong (@noppanut15) <noppanut.connect@gmail.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_site_anp_epg_staticport
short_description: Manage site-local EPG static ports in schema template
description:
- Manage site-local EPG static ports in schema template on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
- Akini Ross (@akinross)
- Noppanut Ploywong (@noppanut15)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  site:
    description:
    - The name of the site.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  anp:
    description:
    - The name of the ANP.
    type: str
    required: true
  epg:
    description:
    - The name of the EPG.
    type: str
    required: true
  type:
    description:
    - The path type of the static port
    - C(vpc) is used for a Virtual Port Channel
    - C(dpc) is used for a Direct Port Channel
    - C(port) is used for a single interface
    type: str
    choices: [ port, vpc, dpc ]
    default: port
  pod:
    description:
    - The pod of the static port.
    type: str
  leaf:
    description:
    - The leaf of the static port.
    type: str
  fex:
    description:
    - The fex id of the static port.
    type: str
  path:
    description:
    - The path of the static port.
    type: str
  vlan:
    description:
    - The port encap VLAN id of the static port.
    type: int
  deployment_immediacy:
    description:
    - The deployment immediacy of the static port.
    - C(immediate) means B(Deploy immediate).
    - C(lazy) means B(deploy on demand).
    type: str
    choices: [ immediate, lazy ]
    default: lazy
  mode:
    description:
    - The mode of the static port.
    - C(native) means B(Access (802.1p)).
    - C(regular) means B(Trunk).
    - C(untagged) means B(Access (untagged)).
    type: str
    choices: [ native, regular, untagged ]
    default: untagged
  primary_micro_segment_vlan:
    description:
    - Primary micro-seg VLAN of static port.
    type: int
  force_replace:
    description:
    - Replaces all the configured static port(s) with the provided static port(s).
    - This option can only be used in combination with the O(static_ports) option.
    - In combination with the O(state=absent) and without any static port configuration all configured static port(s) will be removed.
    type: bool
  static_ports:
    description:
    - A list of Static Ports associated to this EPG.
    - All configured Static Ports will be replaced with the provided Static Ports when used with O(force_replace=true).
    - Only the provided Static Ports will be added, updated or removed when used with O(force_replace=false).
    - In combination with the O(state=query) all provided Static Ports must be found else the task will fail.
    - When I(static_ports) attributes are not provided the module attributes will be used.
    - For each Static Ports provided in the list, the following attributes must be resolved
    - I(static_ports.type)
    - I(static_ports.pod)
    - I(static_ports.leaf)
    - I(static_ports.path)
    - I(static_ports.vlan)
    type: list
    elements: dict
    suboptions:
      type:
        description:
        - The path type of the static port
        - C(vpc) is used for a Virtual Port Channel
        - C(dpc) is used for a Direct Port Channel
        - C(port) is used for a single interface
        type: str
        choices: [ port, vpc, dpc ]
      pod:
        description:
        - The pod of the static port.
        type: str
      leaf:
        description:
        - The leaf of the static port.
        type: str
      fex:
        description:
        - The fex id of the static port.
        type: str
      path:
        description:
        - The path of the static port.
        type: str
      vlan:
        description:
        - The port encapsulation VLAN id of the static port.
        type: int
      deployment_immediacy:
        description:
        - The deployment immediacy of the static port.
        - C(immediate) means B(Deploy immediate).
        - C(lazy) means B(Deploy on demand).
        type: str
        choices: [ immediate, lazy ]
      mode:
        description:
        - The mode of the static port.
        - C(native) means B(Access (802.1p)).
        - C(regular) means B(Trunk).
        - C(untagged) means B(Access (untagged)).
        type: str
        choices: [ native, regular, untagged ]
      primary_micro_segment_vlan:
        description:
        - Primary micro-seg VLAN of static port.
        type: int
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- The ACI MultiSite PATCH API has a deficiency requiring some objects to be referenced by index.
  This can cause silent corruption on concurrent access when changing/removing an object as
  the wrong object may be referenced. This module is affected by this deficiency.
seealso:
- module: cisco.mso.mso_schema_site_anp_epg
- module: cisco.mso.mso_schema_template_anp_epg
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new static port to a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    path: eth1/1
    vlan: 126
    deployment_immediacy: immediate
    state: present

- name: Add a new static fex port to a site EPG
  mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    fex: 151
    path: eth1/1
    vlan: 126
    deployment_immediacy: lazy
    state: present

- name: Add a new static VPC to a site EPG
  mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    pod: pod-1
    leaf: 101-102
    path: ansible_polgrp
    vlan: 127
    type: vpc
    mode: untagged
    deployment_immediacy: lazy
    state: present

- name: Add two new static port to a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    static_ports:
      - pod: pod-1
        leaf: 101
        path: eth1/1
        vlan: 126
      - pod: pod-2
        leaf: 101
        path: eth2/1
        vlan: 128
    deployment_immediacy: immediate
    state: present

- name: Replace all existing static pors on a site EPG with 2 new static ports
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    force_replace: true
    static_ports:
      - pod: pod-1
        leaf: 101
        path: eth1/1
        vlan: 126
      - pod: pod-2
        leaf: 101
        path: eth2/1
        vlan: 128
    deployment_immediacy: immediate
    state: present

- name: Query a specific site EPG static port
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    path: eth1/1
    state: query
  register: query_result

- name: Query a list of static ports
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    static_ports:
      - pod: pod-1
        leaf: 101
        path: eth1/1
        vlan: 126
      - pod: pod-2
        leaf: 101
        path: eth2/1
        vlan: 128
    state: query
  register: query_result

- name: Query all site EPG static ports
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    state: query
  register: query_result

- name: Remove a static port from a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    path: eth1/1
    state: absent

- name: Remove two static ports from a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    static_ports:
      - pod: pod-1
        leaf: 101
        path: eth1/1
        vlan: 126
      - pod: pod-2
        leaf: 101
        path: eth2/1
        vlan: 128
    deployment_immediacy: immediate
    state: absent

- name: Remove all existing static pors from a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    force_replace: true
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_site_anp_epg_bulk_staticport_spec
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        site=dict(type="str", required=True),
        template=dict(type="str", required=True),
        anp=dict(type="str", required=True),
        epg=dict(type="str", required=True),
        force_replace=dict(type="bool"),
        type=dict(type="str", default="port", choices=["port", "vpc", "dpc"]),
        pod=dict(type="str"),
        leaf=dict(type="str"),
        fex=dict(type="str"),
        path=dict(type="str"),
        vlan=dict(type="int"),
        primary_micro_segment_vlan=dict(type="int"),
        deployment_immediacy=dict(type="str", default="lazy", choices=["immediate", "lazy"]),
        mode=dict(type="str", default="untagged", choices=["native", "regular", "untagged"]),
        static_ports=dict(type="list", elements="dict", options=mso_site_anp_epg_bulk_staticport_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    schema = module.params.get("schema")
    site = module.params.get("site")
    template = module.params.get("template").replace(" ", "")
    anp = module.params.get("anp")
    epg = module.params.get("epg")
    path_type = module.params.get("type")
    pod = module.params.get("pod")
    leaf = module.params.get("leaf")
    fex = module.params.get("fex")
    path = module.params.get("path")
    vlan = module.params.get("vlan")
    primary_micro_segment_vlan = module.params.get("primary_micro_segment_vlan")
    deployment_immediacy = module.params.get("deployment_immediacy")
    mode = module.params.get("mode")
    force_replace = module.params.get("force_replace")
    static_ports = module.params.get("static_ports")
    state = module.params.get("state")

    if not static_ports and state in ["present", "absent"]:
        if state == "absent":
            # The state absent requires at least the pod, leaf and path to be provided
            key_list = ["pod", "leaf", "path"]
        else:
            # The state present requires all the key_list to be provided
            key_list = ["pod", "leaf", "path", "vlan"]
        required_missing = [key for key in key_list if module.params.get(key)]
        if len(required_missing) != len(key_list) and not (len(required_missing) == 0 and state == "absent" and force_replace):
            module.fail_json(
                msg="state is present or absent but all of the following are missing: {0}.".format(
                    ", ".join([key for key in key_list if not module.params.get(key)]),
                )
            )

    mso = MSOModule(module)

    mso_schema = MSOSchema(mso, schema, template, site)
    mso_schema.set_template_anp(anp)
    mso_schema.set_template_anp_epg(epg)
    mso_schema.set_site_anp(anp, False)
    mso_schema.set_site_anp_epg(epg, False)

    ops = []

    # Create missing site anp and site epg if not present
    # Logic is only needed for NDO version below 4.x when validate false flag was still available
    # This did not trigger the auto creation of site anp and site epg during template anp and epg creation or ataching site to template
    # Coverage misses this two conditionals when testing on 4.x and above
    if state == "present" and not mso_schema.schema_objects.get("site_anp"):
        ops.append(
            dict(
                op="add",
                path="/sites/{0}-{1}/anps/-".format(mso_schema.schema_objects.get("site").details.get("siteId"), template),
                value=dict(epgRef=dict(schemaId=mso_schema.id, templateName=template, anpName=anp, epgName=epg)),
            )
        )

    if state == "present" and not mso_schema.schema_objects.get("site_anp_epg"):
        ops.append(
            dict(
                op="add",
                path="/sites/{0}-{1}/anps/{2}/epgs/-".format(mso_schema.schema_objects.get("site").details.get("siteId"), template, anp),
                value=dict(anpRef=dict(schemaId=mso_schema.id, templateName=template, anpName=anp)),
            )
        )

    static_ports_path = "/sites/{0}-{1}/anps/{2}/epgs/{3}/staticPorts".format(
        mso_schema.schema_objects.get("site").details.get("siteId"),
        template,
        anp,
        epg,
    )
    static_port_path = "{0}/-".format(static_ports_path)

    full_paths = []
    if static_ports:
        found_static_ports = []
        found_full_paths = []
        set_existing_static_ports(mso, mso_schema, full_paths)
        for static_port in static_ports:
            overwrite_static_path_unprovided_attributes(
                state, mso, static_port, path_type, pod, leaf, fex, path, vlan, primary_micro_segment_vlan, deployment_immediacy, mode
            )
            full_path = get_full_static_path(
                static_port.get("type"), static_port.get("pod"), static_port.get("leaf"), static_port.get("fex"), static_port.get("path")
            )
            mso_schema.set_site_anp_epg_static_port(full_path, False)
            if mso_schema.schema_objects.get("site_anp_epg_static_port") is not None:
                found_static_ports.append(mso_schema.schema_objects["site_anp_epg_static_port"].details)
                found_full_paths.append(full_path)

    elif (path_type and pod and leaf and path) and (vlan or (state == "absent")):
        full_path = get_full_static_path(path_type, pod, leaf, fex, path)
        mso_schema.set_site_anp_epg_static_port(full_path, False)
        if mso_schema.schema_objects.get("site_anp_epg_static_port") is not None:
            mso.existing = mso_schema.schema_objects["site_anp_epg_static_port"].details
            static_port_path = "{0}/{1}".format(static_ports_path, mso_schema.schema_objects["site_anp_epg_static_port"].index)
    else:
        set_existing_static_ports(mso, mso_schema, full_paths)

    if state == "query":
        if static_ports:
            if len(found_static_ports) == len(static_ports):
                mso.existing = found_static_ports
            else:
                configured_static_ports = [
                    get_full_static_path(
                        static_port.get("type"), static_port.get("pod"), static_port.get("leaf"), static_port.get("fex"), static_port.get("path")
                    )
                    for index, static_port in enumerate(static_ports)
                ]
                not_found_static_ports = [
                    "Provided Static Port Path '{0}' not found".format(static_port)
                    for static_port in configured_static_ports
                    if static_port not in found_full_paths
                ]
                mso.fail_json(msg=not_found_static_ports)
        elif not mso.existing and full_path:
            mso.fail_json(msg="Provided Static Port Path '{0}' not found".format(full_path))
        mso.exit_json()

    mso.previous = mso.existing

    if state == "absent" and mso.existing:
        if static_ports and not force_replace:
            mso.proposed = mso.existing.copy()
            remove_index = []
            for found_full_path in found_full_paths:
                if found_full_path in full_paths:
                    index = full_paths.index(found_full_path)
                    remove_index.append(index)
            # The list index should not shift when removing static ports from the list
            # By sorting the indexes found in reverse order, we assure that the highest index is removed first by the NDO backend
            # This logic is to avoid removing the wrong static ports
            for index in reversed(sorted(remove_index)):
                mso.proposed.pop(index)
                ops.append(dict(op="remove", path="{0}/{1}".format(static_ports_path, index)))
            mso.sent = mso.proposed
        elif not force_replace:
            mso.sent = mso.existing = {}
            ops.append(dict(op="remove", path=static_port_path))
        else:
            mso.sent = mso.proposed = mso.existing = []
            ops.append(dict(op="remove", path=static_ports_path))

    elif state == "present":
        if static_ports and force_replace:
            mso.sent = mso.proposed = [
                get_static_port_payload(
                    get_full_static_path(
                        static_port.get("type"),
                        static_port.get("pod"),
                        static_port.get("leaf"),
                        static_port.get("fex"),
                        static_port.get("path"),
                    ),
                    static_port.get("deployment_immediacy"),
                    static_port.get("mode"),
                    static_port.get("vlan"),
                    static_port.get("type"),
                    static_port.get("primary_micro_segment_vlan"),
                )
                for static_port in static_ports
            ]
            if mso.existing:
                ops.append(dict(op="replace", path=static_ports_path, value=mso.sent))
            else:
                ops.append(dict(op="add", path=static_ports_path, value=mso.sent))
        elif static_ports:
            mso.sent = mso.proposed = mso.existing.copy()
            for static_port in static_ports:
                full_path = get_full_static_path(
                    static_port.get("type"), static_port.get("pod"), static_port.get("leaf"), static_port.get("fex"), static_port.get("path")
                )
                payload = get_static_port_payload(
                    full_path,
                    static_port.get("deployment_immediacy"),
                    static_port.get("mode"),
                    static_port.get("vlan"),
                    static_port.get("type"),
                    static_port.get("primary_micro_segment_vlan"),
                )
                if full_path not in found_full_paths:
                    ops.append(dict(op="add", path=static_port_path, value=payload))
                    mso.proposed.append(payload)
                else:
                    index = full_paths.index(full_path)
                    mso.proposed[index] = payload
                    ops.append(dict(op="replace", path="{0}/{1}".format(static_ports_path, index), value=payload))
        else:
            payload = get_static_port_payload(full_path, deployment_immediacy, mode, vlan, path_type, primary_micro_segment_vlan)
            mso.sanitize(payload, collate=True)
            if mso.existing:
                ops.append(dict(op="replace", path=static_port_path, value=mso.sent))
            else:
                ops.append(dict(op="add", path=static_port_path, value=mso.sent))
            mso.existing = payload

    mso.existing = mso.proposed

    if not module.check_mode and mso.proposed != mso.previous:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


def set_existing_static_ports(mso, mso_schema, full_paths):
    mso.existing = []
    if mso_schema.schema_objects.get("site_anp_epg"):
        for existing_static_port in mso_schema.schema_objects["site_anp_epg"].details.get("staticPorts"):
            full_paths.append(existing_static_port.get("path"))
            mso.existing.append(existing_static_port)


def get_static_port_payload(full_path, deployment_immediacy, mode, vlan, path_type, primary_micro_segment_vlan):
    payload = dict(
        deploymentImmediacy=deployment_immediacy,
        mode=mode,
        path=full_path,
        portEncapVlan=vlan,
        type=path_type,
    )
    if primary_micro_segment_vlan:
        payload.update(microSegVlan=primary_micro_segment_vlan)
    return payload


def get_full_static_path(path_type, pod, leaf, fex, path):
    if path_type == "port" and fex is not None:
        return "topology/{0}/paths-{1}/extpaths-{2}/pathep-[{3}]".format(pod, leaf, fex, path)
    elif path_type == "vpc":
        return "topology/{0}/protpaths-{1}/pathep-[{2}]".format(pod, leaf, path)
    else:
        return "topology/{0}/paths-{1}/pathep-[{2}]".format(pod, leaf, path)


def overwrite_static_path_unprovided_attributes(state, mso, static_path, path_type, pod, leaf, fex, path, vlan, micro_vlan, deployment_immediacy, mode):
    required_overwrites = []
    if not static_path.get("type"):
        static_path["type"] = path_type
    if not static_path.get("pod"):
        static_path["pod"] = pod
        if not pod:
            required_overwrites.append("pod")
    if not static_path.get("leaf"):
        static_path["leaf"] = leaf
        if not leaf:
            required_overwrites.append("leaf")
    if not static_path.get("fex"):
        static_path["fex"] = fex
    if not static_path.get("path"):
        static_path["path"] = path
        if not path:
            required_overwrites.append("path")
    if not static_path.get("vlan"):
        static_path["vlan"] = vlan
        if not vlan and state != "absent":
            # The vlan is not required when the state is absent
            required_overwrites.append("vlan")
    if not static_path.get("primary_micro_segment_vlan"):
        static_path["primary_micro_segment_vlan"] = micro_vlan
    if not static_path.get("deployment_immediacy"):
        static_path["deployment_immediacy"] = deployment_immediacy
    if not static_path.get("mode"):
        static_path["mode"] = mode

    if required_overwrites:
        mso.fail_json(msg="state is present but all of the following are missing: {0}.".format(", ".join(required_overwrites)))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    # Standard files documentation fragment
    DOCUMENTATION = r"""
options:
  ptp:
    description:
    - The Precision Time Protocol (PTP) configuration for the interface.
    - Providing an empty dictionary O(ptp={}) will remove the PTP configuration from the interface.
    type: dict
    suboptions:
      mode:
        description:
        - The PTP mode.
        - If this parameter is unspecified, NDO defaults to O(ptp.mode=multicast_dynamic).
        type: str
        choices: [ multicast_dynamic, multicast_master, unicast_master ]
      source_address:
        description:
        - The PTP source address.
        - If this parameter is unspecified, NDO defaults to O(ptp.source_address=0.0.0.0).
        type: str
      user_profile:
        description:
        - The PTP user profile.
        type: dict
        suboptions:
          uuid:
            description:
            - The UUID of the PTP user profile.
            type: str
          reference:
            description:
            - The reference details of the PTP user profile.
            type: dict
            aliases: [ ref ]
            suboptions:
              name:
                description:
                - The name of the PTP user profile.
                type: str
                required: true
              template:
                description:
                - The name of the template that contains the PTP user profile.
                - This parameter or O(ptp.user_profile.reference.template_id) is required.
                type: str
              template_id:
                description:
                - The ID of the template that contains the PTP user profile.
                - This parameter or O(ptp.user_profile.reference.template) is required.
                type: str
      unicast_destinations:
        description:
        - The PTP unicast destination IP addresses.
        - The old O(ptp.unicast_destinations) will be replaced with the new O(ptp.unicast_destinations) during an update.
        - The PTP unicast destinations IP addresses can only be configured if O(ptp.mode=unicast_master).
        type: list
        elements: str
"""
